#!/bin/sh
#
#  Copyright IBM Corp. 2015-2017
#  All rights reserved.
#  US Government Users Restricted Rights -
#  Use, duplication or disclosure restricted
#  by GSA ADP Schedule Contract with IBM Corp.
#
#  Licensed Materials-Property of IBM
#
echo "Running JDK upgrade script --- installServiceforCB.sh"
dos2unix *
####################################################################
# Get saved values. Anything in service.properties will override
# Source the WHV virtualimage.properties file
####################################################################
. /etc/virtualimage.properties

if  [ "" != "$WAS_BITNESS" ]; then
    BITNESS=$WAS_BITNESS
    echo "Found bitness in source env=$WAS_BITNESS"
elif [ -r $PROFILES_HOME/.ibm/bitness ]; then
    BITNESS=`cat $PROFILES_HOME/.ibm/bitness`
    echo "Found bitness file:$PROFILES_HOME/.ibm/bitness  set bitness to: $BITNESS"
else
    echo "bitness file: $PROFILES_HOME/.ibm/bitness not found trying system bitness check"
    $WAS_HOME/bin/versionInfo.sh |grep "32 bit"  >/dev/null
    bit32result=$?
    $WAS_HOME/bin/versionInfo.sh |grep "64 bit" >/dev/null
    bit64result=$?
    if [ $bit64result == 0 ] ; then 
        BITNESS="64"
    elif [ $bit32result == 0 ] ; then 
        BITNESS="32"
    else 
       echo "Dumping bitness command = `$WAS_HOME/bin/versionInfo.sh |grep "bit"`  "
       echo "ERROR: Unable to determine system bitness"
       exit 1
    fi 
    echo "bitness returned $BITNESS"

fi
####################################################################
# Determine arch Linix | AIX
###################################################################
if [ -f /etc/SuSE-release ] || [ -f /etc/redhat-release ] ; then
	ARCH="LinuxX$BITNESS"
    echo "The target system is LinuxX$BITNESS"
elif [ -f /etc/motd ]; then
    grep -q AIX /etc/motd
    result=$?
    if [ $result == "0" ] ; then
    	ARCH="AixPPC$BITNESS"
        echo "The target system is AixPPC$BITNESS"
    fi
else
    echo "ERROR: The target operating system is not supported by this interim fix"
    exit 1
fi
####################################################################
# source the properties, with bitness and arch replaced with values above 
###################################################################

if [ -f ./service.properties ]; then
    . ./service.properties
else
	echo "ERROR: Cannot read service.properties."
	exit 1
fi
####################################################################
# Determine proper Userid for script
###################################################################

#Default the install user if not saved 
if  [ "" == "$INSTALL_USER" ]; then
    INSTALL_USER="virtuser"
fi

# WHV_USER is the new value, but we can always find it in username
if [ -z "$WHV_USER" ]; then
    if [ -r $PROFILES_HOME/.ibm/username ]; then
        WHV_USER=`cat $PROFILES_HOME/.ibm/username`
    else
        WHV_USER="virtuser"
    fi
fi

####################################################################
# Setup IM installer Repository URL
####################################################################

# Default the next two if not already specified
if [ -z "$IM_HOME" ]; then
    IM_HOME="/home/$INSTALL_USER/IBM/InstallationManager/eclipse/"
fi
if [ -z "$REPOSITORY_LOCATION" ]; then
    REPOSITORY_LOCATION="http://IBMWorkloadDeployer:8585/IMRepository/Compo"
fi

####################################################################
# Setup IM installer Credential variables
####################################################################
SERVICE_DIR=`pwd`
chmod 777 $SERVICE_DIR
if [ -f $IM_HOME/tools/master_password_file.txt ]; then
    echo "InstallationManager is configured with credentials"
    CREDENTIALS="-secureStorageFile $IM_HOME/tools/credential.store -masterPasswordFile $IM_HOME/tools/master_password_file.txt"
else
    echo "InstallationManager is NOT configured with credentials"
    CREDENTIALS=""
fi

#####################################################################
# List current installed packages 
#####################################################################
   echo "============================="
   su $INSTALL_USER -c "$IM_HOME/tools/imcl listInstalledPackages -long >$SERVICE_DIR/installed.log"
   STEP_RESULT=$?
   if [ $? -ne 0 ]; then
       echo "ERROR: InstallationManager to listInstalledPackages: return code $?"
       echo "STEP_RESULT: $STEP_RESULT"
       exit 1
   fi
   echo "Currently installed packages:"
   cat "$SERVICE_DIR/installed.log"
   echo "============================="

####################################################################
# List Available fix packs that are currently are available in the repo
#
#  Parms:
#     $1=package offering id
#     $2=package offering service version 
#     $3=Generic package query to force imcl to find ALL version fixes 
#  Script will exit rc=1 if the IFIX service version package
#  is not available in the repository

####################################################################
find_ifix_repo()
{
   IPACKAGE=$1
   IVERSION=$2
   IFIXQUERY=$3  
   echo "============================="
   echo "Running imcl IFIX repo search please wait ...................."
   
   #VAR+= allows a var to be concatenated  str1+str1+str3 etc.
#   COMBINESTR=$IPACKAGE
#   COMBINESTR+="_"
#   COMBINESTR+=$IFIXQUERY
   echo "Cmd Str: su $INSTALL_USER -c $IM_HOME/tools/imcl listAvailableFixes ${IPACKAGE}_${IFIXQUERY}  $CREDENTIALS  -repositories $REPOSITORY_LOCATION -long >$SERVICE_DIR/iFIXrepo.log"
   su $INSTALL_USER -c "$IM_HOME/tools/imcl listAvailableFixes ${IPACKAGE}_${IFIXQUERY}  $CREDENTIALS  -repositories $REPOSITORY_LOCATION -long >$SERVICE_DIR/iFIXrepo.log"
   STEP_RESULT=$?
   if [ $? -ne 0 ]; then
       echo "ERROR: InstallationManager listAvailableFixes Query failed return code $?"
       echo "STEP_RESULT: $STEP_RESULT"
       exit 1
   fi
   echo "The installation repository contains the following packages:"
   cat  $SERVICE_DIR/iFIXrepo.log
   cat  $SERVICE_DIR/iFIXrepo.log >>$SERVICE_DIR/appendRepo.log
   echo "============================="
}

#####################################################################
# java running checklifecycle 
#
#  No parms: Check if JDK have stopped or give additional time 30 sec.
#####################################################################
java_running_delay()
{

   # Check if a WAS Java process is running
   ps -ef|grep java > /tmp/wasjavaps
   grep "$WAS_HOME/java/bin/java" /tmp/wasjavaps > /dev/null 2>&1
   isWASJAVARunning=$?
   if [ $isWASJAVARunning -eq 0 ]; then
      echo "Delaying 30 seconds to allow java processes to exit"
      sleep 30
   fi
   ps -ef|grep java > /tmp/wasjavaps
   grep "$WAS_HOME/java/bin/java" /tmp/wasjavaps > /dev/null 2>&1
   isWASJAVARunning=$?
   rm -f /tmp/wasjavaps
   if [ $isWASJAVARunning -eq 0 ]; then
      echo "ERROR: A WAS Java process is running, please stop it and rerun this script to do upgrade."
      exit 1
   fi
}

#####################################################################
# IHS lifecycle 
#
# $1= Stop / Start 
#####################################################################
ihs_lifecycle()
{
   ISERVICE=$1
   SKIP="FALSE" 

   if [ -z "$IHS" ]; then
      echo "IHS configuration not complete - skipping IHS lifeCycle process"
      return 0
   fi

   #Ensure we only stop or start ALL servers once
   if [ "$ISERVICE" == "STOP" ]; then
      if [ "$STOP_SKIP" == "TRUE" ]; then
         SKIP="TRUE"
      fi
   else  
      if [ "$START_SKIP" == "TRUE" ]; then
         SKIP="TRUE"
      fi
   fi 

   if [ "$SKIP" == "TRUE" ]; then
      echo "Already $ISERVICE all servers skipping IHS lifeCycle process "
   else

      #  IHS. Ignore errors, because server may already be stopped
      echo "============================="
      if [ -f "$IHS_HOME/bin/apachectl" ]; then
         echo "ISERVICE IHS"
         if [ "$ISERVICE" == "STOP" ]; then
            $IHS_HOME/bin/apachectl stop
            $IHS_HOME/bin/adminctl stop
            # Give 30 seconds for java to process to settle.
            java_running_delay
         else  
            $IHS_HOME/bin/apachectl start
            $IHS_HOME/bin/adminctl start
         fi 
      fi
   fi
}

#####################################################################
# Liberty lifecycle 
#
# $1= Stop / Start 
#####################################################################
Liberty_lifecycle()
{
   ISERVICE=$1

   SKIP="FALSE" 
   #Ensure we only stop or start ALL servers once
   if [ "$ISERVICE" == "STOP" ]; then
      if [ "$STOP_SKIP" == "TRUE" ]; then
         SKIP="TRUE"
      fi
   else  
      if [ "$START_SKIP" == "TRUE" ]; then
         SKIP="TRUE"
      fi
   fi 

   if [ "$SKIP" == "TRUE" ]; then
      echo "Already $ISERVICE all servers skipping Liberty lifeCycle process "
   else
      echo "============================="
      echo "ISERVICE Liberty"
      #Liberty. Ignore errors, because server may already be stopped or server not created
      if [ -d "$PROFILES_HOME/Liberty/servers" ]; then
         cd "$PROFILES_HOME/Liberty/servers"
         for server in `ls -d *`; do
            echo "$ISERVICE Liberty server $server"
            echo "If configuration has not completed an error will be displayed and ignored"
            if [ "$ISERVICE" == "STOP" ]; then
               su $WHV_USER -c "$WLP_HOME/bin/server stop $server"
            else 
               su $WHV_USER -c "$WLP_HOME/bin/server start $server"
            fi
         done
         cd "$SERVICE_DIR"
      fi
   fi
}

#####################################################################
# WebSphere lifecycle 
#
# $1= Stop / Start 
#####################################################################

websphere_lifecycle()
{
   ISERVICE=$1
   SKIP="FALSE"
  
   #Ensure we only stop or start ALL servers once
   if [ "$ISERVICE" == "STOP" ]; then
      if [ "$STOP_SKIP" == "TRUE" ]; then
         SKIP="TRUE"
      fi
   else  
      if [ "$START_SKIP" == "TRUE" ]; then
         SKIP="TRUE"
      fi
   fi 
   # Ignore errors, because server may already be stopped or profile not created

   if [ "$SKIP" == "TRUE" ]; then
      echo "Already $ISERVICE all servers skipping WebSphere lifeCycle process "
   else 
      echo "============================="
      echo " Running WebSphere lifecycle script $ISERVICE please wait ..................."  

      for profile in `ls -d $PROFILES_HOME/*/`; do
          if [ -f "$profile/bin/stopServer.sh" ]; then
              echo "$ISERVICE servers in profile $profile"
              cd "$profile/servers"
              for server in `ls -d *`; do
                  echo $server | grep "webserver" >/dev/null
                  if [ $? -ne 0 ]; then
                     echo "If configuration has not completed an error will be displayed and ignored"
                     if [ "$PROCESS_RUNAS_ROOT" == "TRUE" ]; then
                        if [ "$ISERVICE" == "STOP" ]; then
                           $profile/bin/stopServer.sh $server
                           STOP_SERVER="TRUE"
                           # Give 30 seconds for java to process to settle.
                           java_running_delay
                        else 
                	         $profile/bin/startServer.sh $server 
                        fi
                     else 
                        if [ "$ISERVICE" == "STOP" ]; then
                           su $WHV_USER -c "$profile/bin/stopServer.sh $server"
                        else
                           su $WHV_USER -c "$profile/bin/startServer.sh $server"
                        fi
              	   fi
                  fi
              done
          fi
      done
   fi
}

####################################################################
# Locate a package in the list of available
#  Parms:
#     $1=package offering id
#     $2=package offering service version 
#  Script will exit rc=1 if the service version 
#  is not available in the repository
####################################################################
findpackage()
{
  IPACKAGE=$1
  IVERSION=$2
  IFIXQUERY=$3

  find_ifix_repo "$IPACKAGE" "$IVERSION" "$IFIXQUERY"

  cat $SERVICE_DIR/iFIXrepo.log | grep $IVERSION >/dev/null
  if [ $? -ne 0 ]; then
      echo "ERROR: Repository does not contain an available iFix Version $IVERSION" | sed 's/\\//g'
      echo "ERROR: The installed package cannot be upgraded $IPACKAGE" | sed 's/\\//g'
      exit 1
  fi
}

####################################################################
# Locate a package in the list of installed
####################################################################
findinstalled()
{
  IPACKAGE=$1

  cat $SERVICE_DIR/installed.log | grep $IPACKAGE >/dev/null
  if [ $? -ne 0 ]; then
      echo "Package $IPACKAGE is NOT currently installed" | sed 's/\\//g'
      return 0
  fi
  return 1
}

####################################################################
#  Check if we need to service this component
#  Parms:
#     $1=package offering id
#     $2=package offering service version (not the currently installed version)
#  Return:
#     0=Do not service this component
#     1=Service this component, and version is available
#  Script will return rc=1 if the package is installed, but the service version 
#  is not available in the repository
####################################################################
servicepackage()
{
    IPACKAGE=$1
    IVERSION=$2
    IFIXQUERY=$3

    # If version not specified, then we will not service this component
    if [ -z "$IVERSION" ]; then
        echo "Version not specified: $IVERSION"
        return 0
    fi

    # Is this package installed on this VM? 
    findinstalled "$IPACKAGE"
    if [ $? -ne 0 ]; then
        echo "Package is installed: $IPACKAGE"
        if [ -z "$IFIXQUERY" ]; then
            echo "This is not a fix, it is a fixpack"
        else
            findpackage "$IPACKAGE" "$IVERSION" "$IFIXQUERY"
        fi

        # If yes, then return 1
        return 1
    else
        # If no just return 0
        return 0
    fi
}

####################################################################
#  Update the java.security file
#  Parms:
#     $1=IM installation dir
#     $2=subdirectory under that for this java
#  Return:
#     0=java directory not specified
####################################################################
updatejava()
{
    IBASE=$1
    IJAVA=$2


    # If IJAVA not specified, then we will not update
    if [ -z "$IJAVA" ]; then
        echo "Updating security file not required for this package."
        return 0
    fi
   
    SFILE="$IBASE/$IJAVA/jre/lib/security/java.security"
    echo "start security update"
    echo "# Inserted by JDK Update eFix"  >>$SFILE
    echo "jdk.tls.disabledAlgorithms=SSLv3, RC4, DH keySize <768, MD5withRSA" >>$SFILE 
    echo "jdk.certpath.disabledAlgorithms=MD2, RSA keySize <1024, MD5" >>$SFILE
    
}

####################################################################
# Uninstall JDK  
#  Parms:
#     $1=package offering id
#     $2=offering service version 
####################################################################
uninstall_jdk(){
   # Imcl creates log file: col 14 contains the previous APAR that needs to be removed.
   INSTALLED_IFIX=$(cat  $SERVICE_DIR/IMupdate.log | grep fix | awk '{print $14}')


   cat $SERVICE_DIR/IMupdate.log | grep $ARCH >/dev/null
   if [ $? -ne 0 ]; then
     echo "No APAR was found to uninstall   "
   else 
      echo "======================================="
      echo " Uninstalling JDK $INSTALLED_IFIX please wait........................."
      echo "su $INSTALL_USER -c $IM_HOME/tools/imcl uninstall $INSTALLED_IFIX -log $SERVICE_DIR/IMuninstall.log"
      su $INSTALL_USER -c "$IM_HOME/tools/imcl uninstall $INSTALLED_IFIX -log $SERVICE_DIR/IMunstall.log"
      STEP_RESULT=$?
      if [ $? -ne 0 ]; then
         echo "ERROR: Uninstall of JDK fail return code $?"
         echo "STEP_RESULT: $STEP_RESULT"
         exit $STEP_RESULT
      fi
      cat "$SERVICE_DIR/IMunstall.log"
      cat  $SERVICE_DIR/IMunstall.log >>$SERVICE_DIR/appendUninstall.log
      echo "STEP_RESULT: $STEP_RESULT"
      echo "============================="
      echo "Verify installed packages after upgrade:"
      su $INSTALL_USER -c "$IM_HOME/tools/imcl listInstalledPackages >$SERVICE_DIR/afterUninstall.log"
      echo "Verify installed packages after uninstall:"
      cat "$SERVICE_DIR/afterUninstall.log"
      echo "Completed uninstall"
      return $STEP_RESULT
   fi

}

####################################################################
# upgrade JDK  
#  Parms:
#     $1=package offering id
#     $2=offering service version 
#     $3= location ifix install directory
#     $4= Java Security some JDKs require manual file updates.  
####################################################################
upgrade_jdk()
{

   IPACKAGE=$1
   IVERSION=$2
   IFIXLOCATION=$3
   IFIXSECURITY=$4
   echo "============================="
   echo " Upgrading JDK please wait ........................"
 
   echo "su $INSTALL_USER -c $IM_HOME/tools/imcl -install $IVERSION -repositories $REPOSITORY_LOCATION -acceptLicense -installationDirectory $IFIXLOCATION -log $SERVICE_DIR/IMupdate.log $CREDENTIALS"
   su $INSTALL_USER -c "$IM_HOME/tools/imcl install $IVERSION -repositories $REPOSITORY_LOCATION -acceptLicense -installationDirectory $IFIXLOCATION -log $SERVICE_DIR/IMupdate.log $CREDENTIALS"
   STEP_RESULT=$?
   cat "$SERVICE_DIR/IMupdate.log"
   su $INSTALL_USER -c "$IM_HOME/tools/imcl listInstalledPackages >$SERVICE_DIR/afterupgrade.log"
   echo "Verify installed packages after Upgrade:"
   cat "$SERVICE_DIR/afterupgrade.log"
   if [ $STEP_RESULT -eq 0 ]; then
      # STEP result good, now validate package
      cat $SERVICE_DIR/afterupgrade.log| grep $IVERSION >/dev/null
      STEP_RESULT=$?
      if [ $STEP_RESULT -ne 0 ]; then
        echo "Validation failed: Package wasn't found in upgrade.log "
       echo "Upgrade Failed: $STEP_RESULT"
       echo "============================="
        return $STEP_RESULT
      else
         echo "Upgraded Passed: $STEP_RESULT"
         echo "============================="
         echo "Check if Java security needs Upgrading"      
         updatejava "$IFIXLOCATION" "$IFIXSECURITY"  
         return $STEP_RESULT
      fi 
   else 
      echo "Upgrade Failed: $STEP_RESULT"
      echo "============================="
      return $STEP_RESULT
   fi
}

####################################################################
# Main Install Code loop - cycle through additional JDK installs
####################################################################

result=0
anyfail=0
FOUND_PAK="FALSE"
STOP_SERVER="FALSE"
START_SERVER="FALSE"
JAVA_SERVICE="FALSE"
SERVICE_DIR=`pwd`
PACKAGE_VALUE=""
NOPAKS="FALSE"

# stop everything just once
websphere_lifecycle "STOP"
Liberty_lifecycle "STOP"
ihs_lifecycle "STOP"
STOP_SKIP="TRUE"
cd $SERVICE_DIR

# Read product file line by line into var $n 
# Example line: JDK70_PACKAGE JDK70_VERSION JDK70_LOCATION JDK70_QUERY JDK70_SECURITY
# While loop uses  braces  {} to control var scope issues. 
cat ./productList.properties  | { while read n
do

   echo "starting Main Upgrade loop"

   # Convert string $n to a sourced var example $JDK70_PACKAGE
   # eval pulls the actual $JDK70_PACKAGE into a local loop var.
   # awk parses the loop string per column see example string above.
   PACKAGE_VALUE=$(echo $n |  awk '{print $1}')
   eval DISP_PACK=\$${PACKAGE_VALUE}
   VERSION_VALUE=$(echo $n |  awk '{print $2}')
   eval DISP_VER=\$${VERSION_VALUE}
   LOCATION_VALUE=$(echo $n |  awk '{print $3}')
   eval DISP_LOC=\$${LOCATION_VALUE}
   PACKAGE_QUERY=$(echo $n |  awk '{print $4}')
   eval DISP_QUERY=\$${PACKAGE_QUERY}
   JAVA_SECURITY=$(echo $n |  awk '{print $5}')
   eval DISP_SECURITY=\$${JAVA_SECURITY}

   servicepackage "$DISP_PACK" "$DISP_VER" "$DISP_QUERY"
   if [ $? -ne 0 ]; then
      #Packages and Repo had required packages so proceed with upgrade process. 
      cat $SERVICE_DIR/installed.log | grep $DISP_PACK | sed "s/ :.*//" > $SERVICE_DIR/location.log
      FOUND_PAK="TRUE"      

      cat $SERVICE_DIR/location.log | while read LOCATION_VALUE
      do
        #declare "${DISP_LOC}"=$LOCATION_TEMP
        upgrade_jdk "$DISP_PACK" "$DISP_VER" "$LOCATION_VALUE" "$DISP_SECURITY"
        result=$?
        if [ $result -ne 0 ]; then
           uninstall_jdk  
           result=$?
           if [ $result -eq 0 ]; then
               echo "Uninstall of previous IFIX successful! JDK only support 1 ifix applied"  
               echo "Retrying upgrade JDK with $DISP_PACK  $DISP_VER ........."
               upgrade_jdk "$DISP_PACK" "$DISP_VER" "$LOCATION_TEMP" "$DISP_SECURITY"
               stepFail=$?
               if [ $stepFail -ne 0 ]; then
                 echo "Error:Remember upgrade JDK with $DISP_PACK  $DISP_VER failed!!"
                 anyfail=$stepFail
               fi 
           fi 
        fi 
      done

   else
      echo   "Upgrade Package $DISP_PACK is not installed"
   fi
done

# Start everything again
websphere_lifecycle "START"
Liberty_lifecycle "START"
ihs_lifecycle "START"
START_SKIP="TRUE"
NOPAKS="TRUE"
cd $SERVICE_DIR

#Determine pass / fail
echo "============================="
if [ "$NOPAKS" == "FALSE" ]; then
    echo "ERROR: No Packages were found to upgrade."
    result=1
fi

if [ $result -eq 0 ]; then
    #Check if there have been any fails
    if [ $anyfail -eq 0 ]; then
       echo "INSTCONFSUCCESS"
    else
       echo "Check logs for possible other JDK fails"
       echo "INSTCONFFAIL"
       result=$anyfail 
    fi 
else
    echo "INSTCONFFAIL"
fi

echo "Clean up" 
cd /tmp/
rm -rf /tmp/update
exit $result
}
